// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tWVP: WORLDVIEWPROJECTION;


///////////TEXTURES/////////////////////////////////////////////////////////////

texture Tex <string uiname="Source1";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);            //apply a texture to the sampler
    MipFilter = LINEAR;            //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture Tex1 <string uiname="Source2";>;
sampler Samp1 = sampler_state     //sampler for doing the texture-lookup
{
    Texture   = (Tex1);           //apply a texture to the sampler
    MipFilter = LINEAR;             //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

texture Tex2 <string uiname="Noise Texture";>;
sampler Samp2 = sampler_state  //sampler for doing the texture-lookup
{
    texture = (Tex2);
    AddressU  = WRAP;           //apply a texture to the sampler
    AddressV  = WRAP;            //sampler states
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float Timer;
float Elapse;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float4 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADER
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{

    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
           
    Out.Pos = mul(PosO, tWVP);
    
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS
// --------------------------------------------------------------------------------------------------

  ////////////////////////////////////////
 // simple interpolation based on time //
////////////////////////////////////////

   float4 Interpolate_PS(vs2ps In) : COLOR0
{
   float t = saturate(Elapse*Timer);

   return lerp(tex2D(Samp, In.TexCd), tex2D(Samp1, In.TexCd), t);
}

  ////////////////////////////////////////
 // fade to black then to next texture //
////////////////////////////////////////

       float4 Fade_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer) * 2 - 1;

       return saturate(-t)*tex2D(Samp, In.TexCd) + saturate(t)*tex2D(Samp1, In.TexCd);
}

  ///////////////////////////////////////////////////////////////////////////////////
 // the image saturates and starts to blend with the next one....then desaturates //
///////////////////////////////////////////////////////////////////////////////////

   float4 Saturate_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer);
       float st = t*2-1;

       float4 cl0 = tex2D(Samp, In.TexCd)*2-1;
       cl0 *= 1+saturate(st+1)*4;
       cl0 = saturate(cl0*0.5+0.5);

       float4 cl1 = tex2D(Samp1, In.TexCd)*2-1;
       cl1 *= 1+saturate(1-st)*4;
       cl1 = saturate(cl1*0.5+0.5);

       return lerp(cl0,cl1,t);
}

  ////////////////////////////////////
 // the image inverts then returns //
////////////////////////////////////

   float4 Invert_PS(vs2ps In) : COLOR0
{
   float t = saturate(Elapse*Timer);
   float ts = t * 2 - 1;

   float4 inv = (1-tex2D(Samp, In.TexCd)) * t + (1-tex2D(Samp1, In.TexCd)) * (1-t);

   return saturate(-ts)*tex2D(Samp, In.TexCd) + inv*(1-abs(ts)) + saturate(ts)*tex2D(Samp1, In.TexCd);
}

  /////////////////////////////////////////////////////////////////////////////
 // the image is turned into rectangles of block color which change in size //
/////////////////////////////////////////////////////////////////////////////

   float4 Pixelate_PS(vs2ps In) : COLOR0
{
   float t = saturate(Elapse*Timer);

// tp becomes a value that is 0 at t=0, 1 at t=0.5 and 0 at t=1.  This function is regularly used
   float tp = t*2;
	 if(tp > 1)
         tp = 2-tp;

// this slows down the transition at the start and the end
	tp *= tp;
	tp *= tp;
	tp = saturate(tp+0.001); // guard against divide by zero

// a displacement is calculated.  this generates the blocks of color.
   float2 disp = frac((In.TexCd.xy-0.5)/tp)-0.5;
	  disp *= -tp;

	t = clamp(t*4-2,-1,1);
	t = t*0.5+0.5;

   return lerp(tex2D(Samp, In.TexCd.xy+disp), tex2D(Samp1, In.TexCd.xy+disp), t);
}

  ////////////////////////////////////////////////////////
 // t represents a line that starts at 0 and ends at 1 //
////////////////////////////////////////////////////////

       float4 WipeHorizontal_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer);
       float s = step(t, In.TexCd.x);

       return tex2D(Samp, In.TexCd)*s + tex2D(Samp1, In.TexCd)*(1-s);
}

       float4 WipeVertical_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer);
       float s = step(t, In.TexCd.y);

       return tex2D(Samp, In.TexCd)*s + tex2D(Samp1, In.TexCd)*(1-s);
}

  ////////////////////////////////
 // the next texture slides in //
////////////////////////////////

    float4 DiagonalSlide_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer);
       float s = t < 1-In.TexCd.x || t < 1-In.TexCd.y;

       return tex2D(Samp, In.TexCd)*s + tex2D(Samp1, frac(In.TexCd+t))*(1-s);
}

  /////////////////////////////////////////////
 // vertical lines fade in the next texture //
/////////////////////////////////////////////

      float numLines;

      float4 VerticalLines_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer)*2-1;
       float x = frac(In.TexCd.x*numLines);
       x *= 4 * (1-x);
       x += t;

       return tex2D(Samp, In.TexCd)*saturate(1-x) + tex2D(Samp1, In.TexCd)*saturate(x);
}

      float4 HorizontalLines_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer)*2-1;
       float y = frac(In.TexCd.y*numLines);
       y *= 4 * (1-y);
       y += t;

       return tex2D(Samp, In.TexCd)*saturate(1-y) + tex2D(Samp1, In.TexCd)*saturate(y);
}
  /////////////////////////
 // stretch the pictures //
/////////////////////////

   float4 StretchHorizontal_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer);

       float4 cl = tex2D(Samp, float2(In.TexCd.x/(1-t), In.TexCd.y))*step(In.TexCd.x/(1-t), 1);
       cl += tex2D(Samp1, float2(1-(1-In.TexCd.x)/t, In.TexCd.y))*step(0, 1-(1-In.TexCd.x)/t);

       return cl;
}

   float4 StretchDiagonal_PS(vs2ps In) : COLOR0
{
   float4 cl;

   float t = saturate(Elapse*Timer);

   float x = lerp(In.TexCd.x, In.TexCd.x/(1-t), In.TexCd.y);
   float y = lerp(In.TexCd.y, In.TexCd.y/(1-t), In.TexCd.x);

	cl = tex2D(Samp, float2(x, y))*step(x, 1)*step(y, 1);

	x = lerp(1-(1-In.TexCd.x)/t, In.TexCd.x, In.TexCd.y);
	y = lerp(1-(1-In.TexCd.y)/t, In.TexCd.y, In.TexCd.x);

	cl += tex2D(Samp1, float2(x, y))*step(0, x)*step(0, y);

   return cl;
}

  //////////////////////////////////////
 // this effect is a bit like a blur //
//////////////////////////////////////

float2 offsets0[4] = {
   -1,  0,
    0,  1,
    1,  0,
    0, -1,
};
float2 offsets1[4] = {
   -1, -1,
   -1,  1,
    1,  1,
    1, -1,
};

       float4 Diverge_PS(vs2ps In) : COLOR0
{
       float t = saturate(Elapse*Timer);
       float tp = t * 4 * (1-t);
	tp *= tp;

       float4 sum = 0;
	for (int n = 0; n < 4; n++)
	{
	sum += tex2D(Samp, In.TexCd.xy + tp * 1.0 * offsets0[n]) * 0.25 * (1-t);
	sum += tex2D(Samp1, In.TexCd.xy + tp * 1.0 * offsets1[n]) * 0.25 * (t);
	}

     return sum;
}

  /////////////////////////////////////////////////
 // The images change in accordence to a spiral //
/////////////////////////////////////////////////

float Exponent;
float Rings;
float SpiraleSpeed;

      float4 Spirale_PS(vs2ps In) : COLOR0
{
      float t = saturate(Elapse*Timer);

	// The spiral will be in the center so the texture coordinate must be signed
     float2 crd = In.TexCd*2-1;

     float ang = atan2(crd.x, crd.y);
     float rad = pow(dot(crd, crd), Exponent);

	// t*t*t*10 makes the transition slow at the start and fast at the end
     float mul = saturate(sin(ang + Rings * rad + t*t*t*10*SpiraleSpeed)*0.5-0.5+t*2);

     return tex2D(Samp, In.TexCd)*(1-mul) + tex2D(Samp1, In.TexCd)*mul;
}

  //////////////////////////////////////////////////
 // lerps between two images one color at a time //
//////////////////////////////////////////////////

   float4 tColor ;

   float4 Color_PS(vs2ps In) : COLOR0
{
   float t = saturate(Elapse*Timer);
   float t3 = t * tColor.a;

   float4 c0 = tex2D(Samp, In.TexCd);
   float4 c1 = tex2D(Samp1, In.TexCd);

   float4 c = 1;

	c.r = c0.r*(1-saturate(t3-tColor.r)) + c1.r*saturate(t3-tColor.r);
	c.g = c0.g*(1-saturate(t3-tColor.g)) + c1.g*saturate(t3-tColor.g);
	c.b = c0.b*(1-saturate(t3-tColor.b)) + c1.b*saturate(t3-tColor.b);

  return c;
}

  ////////////////////////////////////////////////////////////////////////////////////////////
 // distorts the original image gradually by a sine wave and combines it with the next one //
////////////////////////////////////////////////////////////////////////////////////////////

float NumWaves;
float Amplitude;

      float4 SineWave_PS(vs2ps In) : COLOR0
{
      float4 cl;

      float t = saturate(Elapse*Timer);

	/*
	float dis = sin(i.tx.x * 2 * 3.141) * i.tx.y;

	o.c0 = tex2D(TransS0, float2(i.tx.x, i.tx.y+dis));*/

    float dis0 = sin(In.TexCd.x * 2 * 3.141 * NumWaves);
    float dis1 = sin(In.TexCd.x * 2 * 3.141 * NumWaves + 3.141);

	cl  = tex2D(Samp, float2(In.TexCd.x, In.TexCd.y*(1-t)+dis0*t*Amplitude))*(1-t);
	cl += tex2D(Samp1, float2(In.TexCd.x, In.TexCd.y*t+dis1*(1-t)*Amplitude))*t;

    return cl;
}

  ////////////////////////////////////////
 // uses noise to blend between images //
////////////////////////////////////////

   float4 SimpleNoise_PS(vs2ps In) : COLOR0
{
   float t = saturate(Elapse*Timer)*2-1;

   float n = saturate(tex2D(Samp2, In.TexCd).a+t);

   return tex2D(Samp, In.TexCd)*(1-n) + tex2D(Samp1, In.TexCd)*n;
}

/////////////////////////////////////////////////////


   float4 Disolve_PS(vs2ps In) : COLOR0
{
   float t = saturate(Elapse*Timer);

   float n = tex2D(Samp2, In.TexCd).a;
   float d = step(n,t);

   return tex2D(Samp, In.TexCd)*(1-d) + tex2D(Samp1, In.TexCd)*d;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Interpolate
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Interpolate_PS();
  }
}

technique Fade
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Fade_PS();
  }
}

technique Saturate
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Saturate_PS();
  }
}

technique Invert
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Invert_PS();
  }
}

technique Pixelate
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Pixelate_PS();
  }
}

technique WipeHorizontal
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 WipeHorizontal_PS();
  }
}

technique WipeVertical
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 WipeVertical_PS();
  }
}

technique DiagonalSlide
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 DiagonalSlide_PS();
  }
}

technique VerticalLines
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 VerticalLines_PS();
  }
}

technique HorizontalLines
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 HorizontalLines_PS();
  }
}

technique StretchHorizontal
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 StretchHorizontal_PS();
  }
}

technique StretchDiagonal
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 StretchDiagonal_PS();
  }
}

technique Diverge
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Diverge_PS();
  }
}

technique Spirale
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Spirale_PS();
  }
}

technique Colored
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Color_PS();
  }
}

technique SineWave
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 SineWave_PS();
  }
}

technique SimpleNoise
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 SimpleNoise_PS();
  }
}

technique Disolve
{
          pass P0
          {
		VertexShader = compile vs_1_1 VS();
		PixelShader =  compile ps_2_0 Disolve_PS();
  }
}

/////////////////////////////////////////////////////////////////////////////////
